@@ -15,4 +15,8 @@ def parse(text_data):
     ppl = PrepopulatedLoader()
     ppl.prepopulate(None, text_data)
     doc = create_document(uri=None, loader=ppl)
-    return doc
\ No newline at end of file
+    return doc
+
+from .options import ParseOptions
+
+__all__ = [ParseOptions, PrepopulatedLoader]
\ No newline at end of file
