@@ -1,13 +1,14 @@
 
 
 import json
-from typing import Optional
+from typing import Optional, Union
 
 from .loader import LoaderBaseClass, FilesystemLoader
 from .parser import Parser
 from .reference import ReferenceDictionary, JsonAnchor
 from .options import ParseOptions, RefResolutionMode
 
+IndexKey = Union[str, int]
 
 class ReferenceResolutionError(Exception):
     pass
@@ -25,18 +26,16 @@ class CircularDependencyError(ReferenceResolutionError):
 
 class DocElement:
 
-    def __init__(self, doc_root, line: int):
+    def __init__(self, doc_root, parent, idx:IndexKey, line: int):
         self._line = line
         self._doc_root = doc_root
+        self._parent = parent
+        self._idx = idx
 
     @property
     def line(self) -> int:
         return self._line
 
-    @line.setter
-    def line(self, value: int):
-        self._line = value
-
     @property
     def uri_line(self):
         return f"{self.root.uri}:{self.line}"
@@ -45,48 +44,52 @@ class DocElement:
     def root(self):
         return self._doc_root
 
+    @property
+    def index(self):
+        return self._idx
+
     def construct(self, data, parent, idx=None, dollar_id=None):
         if dollar_id is None:
             dollar_id = JsonAnchor.empty()
 
         if isinstance(data, dict):
             if self.root._dollar_ref_token in data and isinstance(data[self.root._dollar_ref_token], str):
-                dref = DocReference(data[self.root._dollar_ref_token], dollar_id, self.root, data.lc.line)
+                dref = DocReference(data[self.root._dollar_ref_token], dollar_id, self.root, parent, idx, data.lc.line)
                 return dref
-            dobj = DocObject(data, self.root, data.lc.line, dollar_id=dollar_id)
+            dobj = DocObject(data, self.root, parent, idx, data.lc.line, dollar_id=dollar_id)
             return dobj
         elif isinstance(data, list):
-            da = DocArray(data, self.root, data.lc.line, dollar_id=dollar_id)
+            da = DocArray(data, self.root, parent, idx, data.lc.line, dollar_id=dollar_id)
             return da
         else:
             if idx is not None:
                 if isinstance(parent, dict):
-                    dv = DocValue.factory(data, self.root, parent.lc.value(idx)[0])
+                    dv = DocValue.factory(data, self.root, parent, idx, parent.lc.value(idx)[0])
                     if dv is not None and not isinstance(dv, bool):
                         dv.set_key(idx, parent.lc.key(idx)[0])
                     return dv
                 elif isinstance(parent, list):
-                    dv = DocValue.factory(data, self.root, parent.lc.item(idx)[0])
+                    dv = DocValue.factory(data, self.root, parent, idx, parent.lc.item(idx)[0])
                     if dv is not None and not isinstance(dv, bool):
                         dv.set_key(idx, parent.lc.item(idx)[0])
                     return dv
             else:
-                return DocValue(data, self.root, line=None)
+                return DocValue(data, self.root, parent, idx, line=None)
 
 
 class DocContainer(DocElement):
 
-    def __init__(self, doc_root, line: int, dollar_id=None):
+    def __init__(self, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int, dollar_id=None):
         if dollar_id is None:
             dollar_id = JsonAnchor.empty()
         self._dollar_id = dollar_id
-        super().__init__(doc_root, line)
+        super().__init__(doc_root, parent, idx, line)
 
 
 class DocObject(DocContainer, dict):
 
-    def __init__(self, data: dict, doc_root, line: int, dollar_id=None):
-        super().__init__(doc_root, line, dollar_id)
+    def __init__(self, data: dict, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int, dollar_id=None):
+        super().__init__(doc_root, parent, idx, line, dollar_id)
         if self.root._dollar_id_token in data:
             self._dollar_id.change_to(data[self.root._dollar_id_token])
             self.root._ref_dictionary.put(self._dollar_id, self)
@@ -96,7 +99,7 @@ class DocObject(DocContainer, dict):
             # In JSON-Schema, these include elements under `enum` and `const`
             # Here we detect those cases and set the tokens to None so we don't
             # detect them.
-            if k in self.root._parse_options.exclude_dollar_id_parse:
+            if self.root._parse_options.should_stop_dollar_id_parse(self._parent, k):
                 self.root._dollar_id_token = None
             if k in self.root._parse_options.exclude_dollar_ref_parse:
                 self.root._dollar_ref_token = None
@@ -104,7 +107,7 @@ class DocObject(DocContainer, dict):
             self[k] = self.construct(data=v, parent=data, idx=k, dollar_id=self._dollar_id.copy())
 
             # Now, restore the $id and $ref parsing tokens
-            if k in self.root._parse_options.exclude_dollar_id_parse:
+            if self.root._parse_options.should_stop_dollar_id_parse(self._parent, k):
                 self.root._dollar_id_token = self.root._parse_options.dollar_id_token
             if k in self.root._parse_options.exclude_dollar_ref_parse:
                 self.root._dollar_ref_token = self.root._parse_options.dollar_ref_token
@@ -119,16 +122,16 @@ class DocObject(DocContainer, dict):
 
 class DocArray(DocContainer, list):
 
-    def __init__(self, data: list, doc_root, line: int, dollar_id=None):
-        super().__init__(doc_root, line, dollar_id)
+    def __init__(self, data: list, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int, dollar_id=None):
+        super().__init__(doc_root, parent, idx, line, dollar_id)
         for i, v in enumerate(data):
             self.append(self.construct(data=v, parent=data, idx=i, dollar_id=self._dollar_id.copy()))
 
 
 class DocReference(DocElement):
 
-    def __init__(self, reference:str, dollar_id:Optional[JsonAnchor], doc_root, line:int):
-        super().__init__(doc_root, line)
+    def __init__(self, reference:str, dollar_id:Optional[JsonAnchor], doc_root: DocElement, parent: DocElement, idx:IndexKey, line:int):
+        super().__init__(doc_root, parent, line, idx)
         self._reference = reference
         self._dollar_id = dollar_id.copy()
         if self._dollar_id is None:
@@ -155,8 +158,8 @@ class DocReference(DocElement):
 
 class DocValue(DocElement):
 
-    def __init__(self, value, doc_root, line: int):
-        DocElement.__init__(self, doc_root, line)
+    def __init__(self, value, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
+        DocElement.__init__(self, doc_root, parent, idx, line)
         self.data = value
         self.key = None
         self.key_line = None
@@ -175,56 +178,56 @@ class DocValue(DocElement):
         return str(self.data)
 
     @staticmethod
-    def factory(value, doc_root, line: int):
+    def factory(value, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
         if isinstance(value, bool):
             return value
         elif isinstance(value, int):
-            return DocInteger(value, doc_root, line)
+            return DocInteger(value, doc_root, parent, idx, line)
         elif isinstance(value, float):
-            return DocFloat(value, doc_root, line)
+            return DocFloat(value, doc_root, parent, idx, line)
         elif isinstance(value, str):
-            return DocString(value, doc_root, line)
+            return DocString(value, doc_root, parent, idx, line)
         elif value is None:
             return None
-        return DocValue(value, doc_root, line)
+        return DocValue(value, doc_root, parent, idx, line)
 
 
 class DocInteger(DocValue, int):
 
-    def __new__(cls, value: int, doc_root, line: int):
+    def __new__(cls, value: int, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
         di = int.__new__(DocInteger, value)
-        di.__init__(value, doc_root, line)
+        di.__init__(value, doc_root, parent, idx, line)
         return di
 
-    def __init__(self, value: int, doc_root, line: int):
-        DocValue.__init__(self, value, doc_root, line)
+    def __init__(self, value: int, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
+        DocValue.__init__(self, value, doc_root, parent, idx, line)
 
 
 class DocFloat(DocValue, float):
 
-    def __new__(cls, value: float, doc_root, line: int):
+    def __new__(cls, value: float, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
         df = float.__new__(DocFloat, value)
-        df.__init__(value, doc_root, line)
+        df.__init__(value, doc_root, parent, idx, line)
         return df
 
-    def __init__(self, value: float, doc_root, line: int):
-        DocValue.__init__(self, value, doc_root, line)
+    def __init__(self, value: float, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
+        DocValue.__init__(self, value, doc_root, parent, idx, line)
 
 
 class DocString(DocValue, str):
 
-    def __new__(cls, value: str, doc_root, line: int):
+    def __new__(cls, value: str, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
         # This is stupid and needs to be fixed.
         # It is here to correctly load a poop emoji found
         # in the minLength.json JSON-Schema test data.
         new_value = json.loads(json.dumps(value))
         new_len = len(new_value)
         ds = str.__new__(DocString, new_value)
-        ds.__init__(new_value, doc_root, line)
+        ds.__init__(new_value, doc_root, parent, idx, line)
         return ds
 
-    def __init__(self, value: str, doc_root, line: int):
-        DocValue.__init__(self, value, doc_root, line)
+    def __init__(self, value: str, doc_root: DocElement, parent: DocElement, idx:IndexKey, line: int):
+        DocValue.__init__(self, value, doc_root, parent, idx, line)
 
 class Document:
     pass
@@ -239,6 +242,8 @@ def create_document(uri, loader: Optional[LoaderBaseClass]=None, options: Option
     base_class = DocObject
     if isinstance(structure, list):
         base_class = DocArray
+    elif isinstance(structure, bool):
+        return structure
     elif isinstance(structure, dict) and options.dollar_ref_token in structure and isinstance(structure[options.dollar_ref_token], str):
         uri = structure[options.dollar_ref_token]
         fragment = None
@@ -248,6 +253,7 @@ def create_document(uri, loader: Optional[LoaderBaseClass]=None, options: Option
         if fragment is not None:
             doc = doc.get_node(fragment)
         return doc
+
     class DocumentRoot(base_class, Document):
 
         def __init__(self, uri, loader: LoaderBaseClass, options: ParseOptions):
@@ -267,7 +273,7 @@ def create_document(uri, loader: Optional[LoaderBaseClass]=None, options: Option
                 new_dollar_id = JsonAnchor.from_string(uri)
                 self._ref_dictionary.put(new_dollar_id, self)
                 self._doc_cache._cache[new_dollar_id] = self
-            super().__init__(structure, self, 0, dollar_id=new_dollar_id)
+            super().__init__(data=structure, doc_root=self, parent=None, idx=None, line=0, dollar_id=new_dollar_id)
             if self._ref_resolution_mode == RefResolutionMode.RESOLVE_REFERENCES:
                 self.resolve_references()
 
